#define _RAM 0x10000000 /* NES internal 2k RAM */
#define _ROM 0x10010000 /* ROM Image */
#define _CODE_BASE 0x12000000 /* translated code */
#define _INT_MAP 0x13000000 /* translation address map */

#define _ZPMEM _RAM
#define _STACK _ZPMEM+0x100 /* Stack at 0100 */
#define _NVRAM _RAM+0x6000 /* Battery RAM (mapped to 6000 in 6502 address space) */
#define _REG1 0x10002000 /* ioregs mapped to 2000 in 6502 memory */
#define _REG2 0x10004000 /* ioregs mapped to 4000 in 6502 memory */
#define _REGI 0x10003000 /* emulator internal registers (not mapped in 6502 space) */
#define _MAPTABLE 0x10003800 /* ROM mapper table (pointers) */
#define _VRAM 0x10008000 /* Video RAM */

#define _VFLAG    _REGI    /* Store overflow flag */
#define _FLAGS    _REGI+4  /* Store 6502 process status reg */
#define _STACKPTR _REGI+8  /* Store 6502 stack pointer */
#define _PCR      _REGI+12 /* Store 6502 program counter */
#define _XPC      _REGI+16 /* Translated program counter */
#define _INRET    _REGI+20 /* Input data return value */
#define _CTNI     _REGI+24 /* Cycles to next interrupt */
#define _LCS      _REGI+28 /* Last count setpoint */
#define _SCANPOS  _REGI+32 /* Current scanline position */
#define _VRAMPTR  _REGI+36 /* address to read/write video memory */ 
#define _DMOD     _REGI+40 /* Dest addr to modify */
#define _LASTBANK _REGI+44 /* Last memory page code executed in */

/* C type definitions */

#ifndef u_char
#define u_char unsigned char
#endif

#define RAM   ((u_char *)(_RAM))   /* NES internal 2k RAM */
#define ZPMEM ((u_char *)(_ZPMEM)) /* Zero page */
#define STACK ((u_char *)(_STACK)) /* stack page */
#define NVRAM ((u_char *)(_NVRAM)) /* Battery RAM */
#define REG1 _REG1
#define REG2 _REG2
#define REGI _REGI
#define MAPTABLE ((u_char **)_MAPTABLE)    /* ROM mapper table (pointers) */
#define ROM       ((u_char *)(_ROM))       /* ROM Image */
#define CODE_BASE ((u_char *)(_CODE_BASE)) /* translated code */
#define INT_MAP ((unsigned int *)_INT_MAP) /* translation address map */
#define VRAM ((u_char *)(_VRAM)) /* Video memory */

#define VFLAG    (*(unsigned int *)(_VFLAG)) /* Store overflow flag */
#define FLAGS    (*(unsigned int *)(_FLAGS)) /* Store 6502 process status reg */
#define STACKPTR (*(unsigned int *)(_STACKPTR)) /* Store 6502 stack pointer */
#define PCR      (*(unsigned int *)(_PCR)) /* Store 6502 program counter */
#define XPC      (*(void **)(_XPC)) /* Translated program counter */
#define INRET    (*(signed int *)(_INRET)) /* Input data return value */
#define CTNI     (*(signed int *)(_CTNI)) /* Cycles to next interrupt */
#define LCS      (*(signed int *)(_LCS)) /* Last count setpoint */
#define SCANPOS  (*(unsigned int *)(_SCANPOS)) /* Current scanline position */
#define VRAMPTR  (*(unsigned int *)(_VRAMPTR)) /* address to read/write video memory */

#define MAXMAPPER 11
